@@ -3,7 +3,7 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional, Type
+from typing import Optional, Type, Dict, Any
 from abc import ABC, abstractmethod
 import sys
 
@@ -12,8 +12,11 @@ from aiohttp import web
 from mautrix.types import RoomID, UserID
 from mautrix.appservice import AppService, ASStateStore
 from mautrix.api import HTTPAPI
+from mautrix import __version__ as __mautrix_version__
 
 from ..util.program import Program
+from ..util.bridge_state import BridgeState, BridgeStateEvent, GlobalBridgeState
+from .commands.manhole import ManholeState
 from .config import BaseBridgeConfig
 from .matrix import BaseMatrixHandler
 from .portal import BasePortal
@@ -47,6 +50,7 @@ class Bridge(Program, ABC):
     repo_url: str
     markdown_version: str
     real_user_content_key: Optional[str] = None
+    manhole: Optional[ManholeState]
 
     def __init__(self, module: str = None, name: str = None, description: str = None,
                  command: str = None, version: str = None,
@@ -61,6 +65,7 @@ class Bridge(Program, ABC):
             self.matrix_class = matrix_class
         if state_store_class:
             self.state_store_class = state_store_class
+        self.manhole = None
 
     def prepare_arg_parser(self) -> None:
         super().prepare_arg_parser()
@@ -107,9 +112,12 @@ class Bridge(Program, ABC):
         self.make_state_store()
         mb = 1024 ** 2
         default_http_retry_count = self.config.get("homeserver.http_retry_count", None)
+        if self.name not in HTTPAPI.default_ua:
+            HTTPAPI.default_ua = f"{self.name}/{self.version} {HTTPAPI.default_ua}"
         self.az = AppService(server=self.config["homeserver.address"],
                              domain=self.config["homeserver.domain"],
                              verify_ssl=self.config["homeserver.verify_ssl"],
+                             connection_limit=self.config["homeserver.connection_limit"],
 
                              id=self.config["appservice.id"],
                              as_token=self.config["appservice.as_token"],
@@ -121,7 +129,7 @@ class Bridge(Program, ABC):
                              bot_localpart=self.config["appservice.bot_username"],
                              ephemeral_events=self.config["appservice.ephemeral_events"],
 
-                             default_ua=f"{self.name}/{self.version} {HTTPAPI.default_ua}",
+                             default_ua=HTTPAPI.default_ua,
                              default_http_retry_count=default_http_retry_count,
 
                              log="mau.as",
@@ -154,12 +162,21 @@ class Bridge(Program, ABC):
         self.log.debug("Starting appservice...")
         await self.az.start(self.config["appservice.hostname"], self.config["appservice.port"])
         await self.matrix.wait_for_connection()
+
         await self.matrix.init_encryption()
         self.add_startup_actions(self.matrix.init_as_bot())
         await super().start()
         self.az.ready = True
 
+        status_endpoint = self.config["homeserver.status_endpoint"]
+        if status_endpoint and await self.count_logged_in_users() == 0:
+            state = BridgeState(state_event=BridgeStateEvent.UNCONFIGURED).fill()
+            await state.send(status_endpoint, self.az.as_token, self.log)
+
     async def stop(self) -> None:
+        if self.manhole:
+            self.manhole.close()
+            self.manhole = None
         await self.az.stop()
         await super().stop()
         if self.matrix.e2ee:
@@ -175,28 +192,62 @@ class Bridge(Program, ABC):
         if user is None:
             return web.json_response({"error": "User not found"}, status=404)
         try:
-            state = await user.get_bridge_state()
+            states = await user.get_bridge_states()
         except NotImplementedError:
             return web.json_response({"error": "Bridge status not implemented"}, status=501)
-        await user.fill_bridge_state(state)
-        return web.json_response(state.serialize())
+        for state in states:
+            await user.fill_bridge_state(state)
+        global_state = BridgeState(state_event=BridgeStateEvent.RUNNING).fill()
+        evt = GlobalBridgeState(remote_states={state.remote_id: state for state in states},
+                                bridge_state=global_state)
+        return web.json_response(evt.serialize())
 
     @abstractmethod
-    async def get_user(self, user_id: UserID, create: bool = True) -> 'BaseUser':
+    async def get_user(self, user_id: UserID, create: bool = True) -> Optional['BaseUser']:
         pass
 
     @abstractmethod
-    async def get_portal(self, room_id: RoomID) -> 'BasePortal':
+    async def get_portal(self, room_id: RoomID) -> Optional['BasePortal']:
         pass
 
     @abstractmethod
-    async def get_puppet(self, user_id: UserID, create: bool = False) -> 'BasePuppet':
+    async def get_puppet(self, user_id: UserID, create: bool = False) -> Optional['BasePuppet']:
         pass
 
     @abstractmethod
-    async def get_double_puppet(self, user_id: UserID) -> 'BasePuppet':
+    async def get_double_puppet(self, user_id: UserID) -> Optional['BasePuppet']:
         pass
 
     @abstractmethod
     def is_bridge_ghost(self, user_id: UserID) -> bool:
         pass
+
+    @abstractmethod
+    async def count_logged_in_users(self) -> int:
+        return 0
+
+    async def manhole_global_namespace(self, user_id: UserID) -> Dict[str, Any]:
+        own_user = await self.get_user(user_id, create=False)
+        try:
+            own_puppet = await own_user.get_puppet()
+        except NotImplementedError:
+            own_puppet = None
+        return {
+            "bridge": self,
+            "manhole": self.manhole,
+            "own_user": own_user,
+            "own_puppet": own_puppet,
+        }
+
+    @property
+    def manhole_banner_python_version(self) -> str:
+        return f"Python {sys.version} on {sys.platform}"
+
+    @property
+    def manhole_banner_program_version(self) -> str:
+        return f"{self.name} {self.version} with mautrix-python {__mautrix_version__}"
+
+    def manhole_banner(self, user_id: UserID) -> str:
+        return (f"{self.manhole_banner_python_version}\n"
+                f"{self.manhole_banner_program_version}\n\n"
+                f"Manhole opened by {user_id}\n")
