@@ -56,7 +56,9 @@ class BaseBridgeConfig(BaseFileConfig, BaseValidatableConfig, ABC):
         copy("homeserver.domain")
         copy("homeserver.verify_ssl")
         copy("homeserver.http_retry_count")
+        copy("homeserver.connection_limit")
         copy("homeserver.status_endpoint")
+        copy("homeserver.message_send_checkpoint_endpoint")
 
         copy("appservice.address")
         copy("appservice.hostname")
@@ -79,6 +81,16 @@ class BaseBridgeConfig(BaseFileConfig, BaseValidatableConfig, ABC):
 
         copy("appservice.ephemeral_events")
 
+        copy("bridge.management_room_text.welcome")
+        copy("bridge.management_room_text.welcome_connected")
+        copy("bridge.management_room_text.welcome_unconnected")
+        copy("bridge.management_room_text.additional_help")
+        copy("bridge.management_room_multiple_messages")
+
+        copy("manhole.enabled")
+        copy("manhole.path")
+        copy("manhole.whitelist")
+
         copy("logging")
 
     @property
