@@ -1,4 +1,4 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -6,21 +6,25 @@
 from typing import Tuple, Optional, Union, TYPE_CHECKING
 import logging
 import asyncio
-import os.path
 import time
-import sys
-
-from yarl import URL
 
 from mautrix.types import (EventID, RoomID, UserID, Event, EventType, MessageEvent, MessageType,
                            MessageEventContent, StateEvent, Membership, MemberStateEventContent,
                            PresenceEvent, TypingEvent, ReceiptEvent, TextMessageEventContent,
                            EncryptedEvent, ReceiptType, SingleReceiptEventContent, StateUnsigned,
-                           MediaRepoConfig)
+                           MediaRepoConfig, BaseRoomEvent, ReactionEvent, RedactionEvent)
 from mautrix.errors import IntentError, MatrixError, MForbidden, DecryptionError, SessionNotFound
 from mautrix.appservice import AppService
+from mautrix.util import markdown
 from mautrix.util.logging import TraceLogger
 from mautrix.util.opt_prometheus import Histogram
+from mautrix.util.message_send_checkpoint import (
+    CHECKPOINT_TYPES,
+    MessageSendCheckpoint,
+    MessageSendCheckpointReportedBy,
+    MessageSendCheckpointStatus,
+    MessageSendCheckpointStep,
+)
 
 from .commands import CommandProcessor
 
@@ -41,7 +45,8 @@ try:
 except ImportError:
     encrypt_attachment = None
 
-EVENT_TIME = Histogram("bridge_matrix_event", "Time spent processing Matrix events", ["event_type"])
+EVENT_TIME = Histogram("bridge_matrix_event", "Time spent processing Matrix events",
+                       ["event_type"])
 
 
 class BaseMatrixHandler:
@@ -77,20 +82,21 @@ class BaseMatrixHandler:
                 bridge=bridge,
                 user_id_prefix=self.user_id_prefix, user_id_suffix=self.user_id_suffix,
                 homeserver_address=self.config["homeserver.address"],
-                db_url=self._get_db_url(bridge.config),
+                db_url=self.config["appservice.database"],
                 key_sharing_config=self.config["bridge.encryption.key_sharing"])
 
-    @staticmethod
-    def _get_db_url(config: 'BaseBridgeConfig') -> str:
-        db_url = config["bridge.encryption.database"]
-        if not db_url or db_url == "default":
-            db_url = config["appservice.database"]
-            parsed_url = URL(db_url)
-            if parsed_url.scheme == "sqlite":
-                # Remove the extension to replace it with our own
-                path = os.path.splitext(parsed_url.path)[0]
-                db_url = f"pickle://{path}.crypto.pickle"
-        return db_url
+        self.management_room_text = self.config.get(
+            "bridge.management_room_text",
+            {
+                "welcome": "Hello, I'm a bridge bot.",
+                "welcome_connected": "Use `help` for help.",
+                "welcome_unconnected": "Use `help` for help on how to log in.",
+            },
+        )
+        self.management_room_multiple_messages = self.config.get(
+            "bridge.management_room_multiple_messages",
+            False,
+        )
 
     async def wait_for_connection(self) -> None:
         self.log.info("Ensuring connectivity to homeserver")
@@ -139,10 +145,6 @@ class BaseMatrixHandler:
 
     async def init_encryption(self) -> None:
         if self.e2ee:
-            if not await self.e2ee.check_server_support():
-                self.log.critical("Encryption enabled in config, but homeserver does not advertise "
-                                  "appservice login")
-                sys.exit(30)
             await self.e2ee.start()
 
     @staticmethod
@@ -198,10 +200,20 @@ class BaseMatrixHandler:
         pass
 
     async def handle_event(self, evt: Event) -> None:
-        """Called by :meth:`int_handle_event` for message events other than m.room.message."""
+        """
+        Called by :meth:`int_handle_event` for message events other than m.room.message.
+
+        **N.B.** You may need to add the event class to :prop:`allowed_event_classes`
+        or override :meth:`allow_matrix_event` for it to reach here.
+        """
 
     async def handle_state_event(self, evt: StateEvent) -> None:
-        """Called by :meth:`int_handle_event` for state events other than m.room.membership."""
+        """
+        Called by :meth:`int_handle_event` for state events other than m.room.membership.
+
+        **N.B.** You may need to add the event class to :prop:`allowed_event_classes`
+        or override :meth:`allow_matrix_event` for it to reach here.
+        """
 
     async def handle_ephemeral_event(self, evt: Union[ReceiptEvent, PresenceEvent, TypingEvent]
                                      ) -> None:
@@ -243,11 +255,31 @@ class BaseMatrixHandler:
         await self.send_welcome_message(room_id, inviter)
 
     async def send_welcome_message(self, room_id: RoomID, inviter: 'BaseUser') -> None:
-        await self.az.intent.send_notice(
-            room_id=room_id,
-            text=f"Hello, I'm a bridge bot. Use `{self.commands.command_prefix} help` for help.",
-            html=f"Hello, I'm a bridge bot. Use <code>{self.commands.command_prefix} help</code> "
-                 f"for help.")
+        has_two_members, bridge_bot_in_room = await self._is_direct_chat(room_id)
+        is_management = has_two_members and bridge_bot_in_room
+
+        welcome_messages = [self.management_room_text.get("welcome")]
+
+        if is_management:
+            if await inviter.is_logged_in():
+                welcome_messages.append(self.management_room_text.get("welcome_connected"))
+            else:
+                welcome_messages.append(self.management_room_text.get("welcome_unconnected"))
+
+            additional_help = self.management_room_text.get("additional_help")
+            if additional_help:
+                welcome_messages.append(additional_help)
+        else:
+            cmd_prefix = self.commands.command_prefix
+            welcome_messages.append(f"Use `{cmd_prefix} help` for help.")
+
+        if self.management_room_multiple_messages:
+            for m in welcome_messages:
+                await self.az.intent.send_notice(room_id, text=m, html=markdown.render(m))
+        else:
+            combined = "\n".join(welcome_messages)
+            combined_html = "".join(map(markdown.render, welcome_messages))
+            await self.az.intent.send_notice(room_id, text=combined, html=combined_html)
 
     async def int_handle_invite(self, room_id: RoomID, user_id: UserID, invited_by: UserID,
                                 event_id: EventID) -> None:
@@ -279,10 +311,28 @@ class BaseMatrixHandler:
 
     async def handle_message(self, room_id: RoomID, user_id: UserID, message: MessageEventContent,
                              event_id: EventID) -> None:
+        async def bail(error_text: str, step=MessageSendCheckpointStep.REMOTE) -> None:
+            self.log.debug(error_text)
+            await MessageSendCheckpoint(
+                event_id=event_id,
+                room_id=room_id,
+                step=step,
+                timestamp=int(time.time() * 1000),
+                status=MessageSendCheckpointStatus.PERM_FAILURE,
+                reported_by=MessageSendCheckpointReportedBy.BRIDGE,
+                event_type=EventType.ROOM_MESSAGE,
+                message_type=message.msgtype,
+                info=error_text
+            ).send(
+                self.log,
+                self.bridge.config["homeserver.message_send_checkpoint_endpoint"],
+                self.az.as_token,
+            )
+
         sender = await self.bridge.get_user(user_id)
         if not sender or not await self.allow_message(sender):
-            self.log.debug(f"Ignoring message {event_id} from {user_id} to {room_id}:"
-                           " User is not whitelisted.")
+            await bail(f"Ignoring message {event_id} from {user_id} to {room_id}:"
+                       " user is not whitelisted.")
             return
         self.log.debug(f"Received Matrix event {event_id} from {sender.mxid} in {room_id}")
         self.log.trace("Event %s content: %s", event_id, message)
@@ -296,14 +346,18 @@ class BaseMatrixHandler:
             if await self.allow_bridging_message(sender, portal):
                 await portal.handle_matrix_message(sender, message, event_id)
             else:
-                self.log.trace("Ignoring event %s from %s: not allowed to send to portal",
-                               event_id, sender.mxid)
+                await bail(f"Ignoring event {event_id} from {sender.mxid}:"
+                           " not allowed to send to portal")
             return
 
         if message.msgtype != MessageType.TEXT:
+            await bail(f"Ignoring event {event_id}: not a portal room and not a m.text message")
             return
         elif not await self.allow_command(sender):
-            self.log.trace("Ignoring command %s from %s", event_id, sender.mxid)
+            await bail(
+                f"Ignoring command {event_id} from {sender.mxid}: not allowed to perform command",
+                step=MessageSendCheckpointStep.COMMAND,
+            )
             return
 
         has_two_members, bridge_bot_in_room = await self._is_direct_chat(room_id)
@@ -317,11 +371,30 @@ class BaseMatrixHandler:
                 # Not enough values to unpack, i.e. no arguments
                 command = text
                 args = []
-            await self.commands.handle(room_id, event_id, sender, command, args, message, portal,
-                                       is_management, bridge_bot_in_room)
+
+            try:
+                await self.commands.handle(room_id, event_id, sender, command, args, message,
+                                           portal, is_management, bridge_bot_in_room)
+            except Exception as e:
+                await bail(repr(e), step=MessageSendCheckpointStep.COMMAND)
+            else:
+                await MessageSendCheckpoint(
+                    event_id=event_id,
+                    room_id=room_id,
+                    step=MessageSendCheckpointStep.COMMAND,
+                    timestamp=int(time.time() * 1000),
+                    status=MessageSendCheckpointStatus.SUCCESS,
+                    reported_by=MessageSendCheckpointReportedBy.BRIDGE,
+                    event_type=EventType.ROOM_MESSAGE,
+                    message_type=message.msgtype,
+                ).send(
+                    self.log,
+                    self.bridge.config["homeserver.message_send_checkpoint_endpoint"],
+                    self.az.as_token,
+                )
         else:
-            self.log.trace("Ignoring event %s from %s: not a command and not a portal room",
-                           event_id, sender.mxid)
+            await bail(f"Ignoring event {event_id} from {sender.mxid}:"
+                       " not a command and not a portal room")
 
     async def _is_direct_chat(self, room_id: RoomID) -> Tuple[bool, bool]:
         try:
@@ -347,11 +420,6 @@ class BaseMatrixHandler:
                                   data: SingleReceiptEventContent) -> None:
         pass
 
-    def filter_matrix_event(self, evt: Event) -> bool:
-        if not isinstance(evt, (MessageEvent, StateEvent, ReceiptEvent)):
-            return False
-        return evt.sender == self.az.bot_mxid
-
     async def try_handle_sync_event(self, evt: Event) -> None:
         try:
             if isinstance(evt, (ReceiptEvent, PresenceEvent, TypingEvent)):
@@ -368,18 +436,22 @@ class BaseMatrixHandler:
 
     async def handle_encrypted(self, evt: EncryptedEvent) -> None:
         if not self.e2ee:
+            self.send_decrypted_checkpoint(evt, "Encryption unsupported", True)
             await self.handle_encrypted_unsupported(evt)
             return
         try:
             decrypted = await self.e2ee.decrypt(evt, wait_session_timeout=5)
         except SessionNotFound as e:
+            self.send_decrypted_checkpoint(evt, e, False)
             await self._handle_encrypted_wait(evt, e, wait=10)
         except DecryptionError as e:
             self.log.warning(f"Failed to decrypt {evt.event_id}: {e}")
             self.log.trace("%s decryption traceback:", evt.event_id, exc_info=True)
+            self.send_decrypted_checkpoint(evt, e, True)
             await self.send_encryption_error_notice(evt, e)
         else:
-            await self.int_handle_event(decrypted)
+            self.send_decrypted_checkpoint(decrypted)
+            await self.int_handle_event(decrypted, send_bridge_checkpoint=False)
 
     async def handle_encrypted_unsupported(self, evt: EncryptedEvent) -> None:
         self.log.debug("Got encrypted message %s from %s, but encryption is not enabled",
@@ -389,29 +461,43 @@ class BaseMatrixHandler:
 
     async def _handle_encrypted_wait(self, evt: EncryptedEvent, err: SessionNotFound, wait: int
                                      ) -> None:
-        self.log.debug(f"Couldn't find session {err.session_id} trying to decrypt {evt.event_id}, waiting even longer")
+        self.log.debug(f"Couldn't find session {err.session_id} trying to decrypt {evt.event_id},"
+                       " waiting even longer")
         msg = ("\u26a0 Your message was not bridged: the bridge hasn't received the decryption "
                f"keys. The bridge will retry for {wait} seconds. If this error keeps happening, "
                "try restarting your client.")
-        event_id = await self.az.intent.send_notice(evt.room_id, msg)
+        try:
+            event_id = await self.az.intent.send_notice(evt.room_id, msg)
+        except IntentError:
+            self.log.debug("IntentError while sending encryption error", exc_info=True)
+            self.log.error("Got IntentError while trying to send encryption error message. "
+                           "This likely means the bridge bot is not in the room, which can "
+                           "happen if you force-enable e2ee on the homeserver without enabling "
+                           "it by default on the bridge (bridge -> encryption -> default).")
+            return
         got_keys = await self.e2ee.crypto.wait_for_session(evt.room_id, err.sender_key,
                                                            err.session_id, timeout=wait)
         if got_keys:
-            self.log.debug(f"Got session {err.session_id} after waiting more, trying to decrypt {evt.event_id} again")
+            self.log.debug(f"Got session {err.session_id} after waiting more, "
+                           f"trying to decrypt {evt.event_id} again")
             try:
                 decrypted = await self.e2ee.decrypt(evt, wait_session_timeout=0)
             except DecryptionError as e:
+                self.send_decrypted_checkpoint(evt, e, True)
                 self.log.warning(f"Failed to decrypt {evt.event_id}: {e}")
                 self.log.trace("%s decryption traceback:", evt.event_id, exc_info=True)
                 msg = f"\u26a0 Your message was not bridged: {e}"
             else:
+                self.send_decrypted_checkpoint(decrypted)
                 await self.az.intent.redact(evt.room_id, event_id)
-                await self.int_handle_event(decrypted)
+                await self.int_handle_event(decrypted, send_bridge_checkpoint=False)
                 return
         else:
-            self.log.warning(f"Didn't get {err.session_id}, giving up on {evt.event_id}")
-            msg = ("\u26a0 Your message was not bridged: the bridge hasn't received the decryption "
-                   "keys. If this error keeps happening, try restarting your client.")
+            error_message = f"Didn't get {err.session_id}, giving up on {evt.event_id}"
+            self.log.warning(error_message)
+            self.send_decrypted_checkpoint(evt, error_message, True)
+            msg = ("\u26a0 Your message was not bridged: the bridge hasn't received the decryption"
+                   " keys. If this error keeps happening, try restarting your client.")
         content = TextMessageEventContent(msgtype=MessageType.NOTICE, body=msg)
         content.set_edit(event_id)
         await self.az.intent.send_message(evt.room_id, content)
@@ -426,12 +512,80 @@ class BaseMatrixHandler:
                 portal.log.debug("Received encryption event in direct portal: %s", evt.content)
                 await portal.enable_dm_encryption()
 
-    async def int_handle_event(self, evt: Event) -> None:
+    def send_message_send_checkpoint(
+        self,
+        evt: Event,
+        step: MessageSendCheckpointStep,
+        err: Optional[Union[Exception, str]] = None,
+        permanent: bool = False,
+    ) -> None:
+        endpoint = self.bridge.config["homeserver.message_send_checkpoint_endpoint"]
+        if not endpoint:
+            return
+        if evt.type not in CHECKPOINT_TYPES:
+            return
+
+        self.log.debug(f"Sending message send checkpoint for {evt.event_id} to API server.")
+        status = MessageSendCheckpointStatus.SUCCESS
+        if err:
+            status = (
+                MessageSendCheckpointStatus.PERM_FAILURE if permanent
+                else MessageSendCheckpointStatus.WILL_RETRY
+            )
+
+        checkpoint = MessageSendCheckpoint(
+            event_id=evt.event_id,
+            room_id=evt.room_id,
+            step=step,
+            timestamp=int(time.time() * 1000),
+            status=status,
+            reported_by=MessageSendCheckpointReportedBy.BRIDGE,
+            event_type=evt.type,
+            message_type=evt.content.msgtype if evt.type == EventType.ROOM_MESSAGE else None,
+            info=str(err) if err else None,
+        )
+        asyncio.create_task(checkpoint.send(self.log, endpoint, self.az.as_token))
+
+    def send_bridge_checkpoint(self, evt: Event) -> None:
+        self.send_message_send_checkpoint(evt, MessageSendCheckpointStep.BRIDGE)
+
+    def send_decrypted_checkpoint(self, evt: Event, err: Optional[Union[Exception, str]] = None,
+                                  permanent: bool = False,) -> None:
+        self.send_message_send_checkpoint(evt, MessageSendCheckpointStep.DECRYPTED, err, permanent)
+
+    allowed_event_classes: Tuple[type, ...] = (
+        MessageEvent, StateEvent, ReactionEvent, EncryptedEvent, RedactionEvent,
+        ReceiptEvent, TypingEvent, PresenceEvent,
+    )
+
+    async def allow_matrix_event(self, evt: Event) -> bool:
+        # If the event is not one of the allowed classes, ignore it.
+        if not isinstance(evt, self.allowed_event_classes):
+            return False
+        # For room events, make sure the message didn't originate from the bridge.
+        if isinstance(evt, BaseRoomEvent):
+            # If the event is from a bridge ghost, ignore it.
+            if evt.sender == self.az.bot_mxid or self.bridge.is_bridge_ghost(evt.sender):
+                return False
+            # If the event is marked as double puppeted and we can confirm that we are in fact
+            # double puppeting that user ID, ignore it.
+            if (
+                evt.content.get(self.az.real_user_content_key, False)
+                and await self.bridge.get_double_puppet(evt.sender) is not None
+            ):
+                return False
+        # For non-room events and non-bridge-originated room events, allow.
+        return True
+
+    async def int_handle_event(self, evt: Event, send_bridge_checkpoint: bool = True) -> None:
         if isinstance(evt, StateEvent) and evt.type == EventType.ROOM_MEMBER and self.e2ee:
             await self.e2ee.handle_member_event(evt)
-        if self.filter_matrix_event(evt):
+        if not await self.allow_matrix_event(evt):
             return
         self.log.trace("Received event: %s", evt)
+
+        if send_bridge_checkpoint:
+            self.send_bridge_checkpoint(evt)
         start_time = time.time()
 
         if evt.type == EventType.ROOM_MEMBER:
