@@ -7,13 +7,16 @@ from .media_repository import MediaRepositoryMethods
 from .misc import MiscModuleMethods
 from .crypto import CryptoMethods
 from .account_data import AccountDataMethods
+from .room_tag import RoomTaggingMethods
+from .push_rules import PushRuleMethods
 
 
-class ModuleMethods(MediaRepositoryMethods, CryptoMethods, AccountDataMethods, MiscModuleMethods):
+class ModuleMethods(MediaRepositoryMethods, CryptoMethods, AccountDataMethods, MiscModuleMethods,
+                    RoomTaggingMethods, PushRuleMethods):
     """
     Methods in section 13 Modules of the spec.
 
     See also: `API reference <https://matrix.org/docs/spec/client_server/r0.6.1.html#modules>`__
     """
 
-    # TODO: subregions 15, 18, 21, 26, 27, others?
+    # TODO: subregions 15, 21, 26, 27, others?
