@@ -3,12 +3,12 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional, List, Union, Dict, Any
+from typing import Optional, List, Sequence, Union, Dict, Any, Callable, Awaitable
 import asyncio
 
 from multidict import CIMultiDict
 
-from mautrix.errors import MatrixResponseError, MatrixRequestError, MRoomInUse
+from mautrix.errors import MatrixResponseError, MatrixRequestError, MRoomInUse, MNotFound
 from mautrix.api import Method, Path
 from mautrix.types import (JSON, UserID, RoomID, RoomAlias, StateEvent, RoomDirectoryVisibility,
                            RoomAliasInfo, RoomCreatePreset, DirectoryPaginationToken, Membership,
@@ -18,7 +18,7 @@ from mautrix.types import (JSON, UserID, RoomID, RoomAlias, StateEvent, RoomDire
 from .events import EventMethods
 from .base import BaseClientAPI
 
-InitialState = List[Union[StateEvent, StrippedStateEvent, Dict[str, Any]]]
+InitialState = Sequence[Union[StateEvent, StrippedStateEvent, Dict[str, Any]]]
 
 
 class RoomMethods(EventMethods, BaseClientAPI):
@@ -150,7 +150,7 @@ class RoomMethods(EventMethods, BaseClientAPI):
             else:
                 raise
 
-    async def remove_room_alias(self, alias_localpart: str) -> None:
+    async def remove_room_alias(self, alias_localpart: str, raise_404: bool = False) -> None:
         """
         Remove a mapping of room alias to room ID.
 
@@ -161,9 +161,15 @@ class RoomMethods(EventMethods, BaseClientAPI):
 
         Args:
             alias_localpart: The room alias to remove.
+            raise_404: Whether 404 errors should be raised as exceptions instead of ignored.
         """
         room_alias = f"#{alias_localpart}:{self.domain}"
-        await self.api.request(Method.DELETE, Path.directory.room[room_alias])
+        try:
+            await self.api.request(Method.DELETE, Path.directory.room[room_alias])
+        except MNotFound:
+            if raise_404:
+                raise
+            # else: ignore
 
     async def get_room_alias(self, room_alias: RoomAlias) -> RoomAliasInfo:
         """
@@ -217,9 +223,9 @@ class RoomMethods(EventMethods, BaseClientAPI):
             The ID of the room the user joined.
         """
         if extra_content:
-            await self.send_member_event(room_id, user_id, Membership.BAN,
+            await self.send_member_event(room_id, self.mxid, Membership.JOIN,
                                          extra_content=extra_content)
-            return
+            return room_id
         content = await self.api.request(Method.POST, Path.rooms[room_id].join, {
             "third_party_signed": third_party_signed,
         } if third_party_signed is not None else None)
@@ -277,11 +283,51 @@ class RoomMethods(EventMethods, BaseClientAPI):
         except KeyError:
             raise MatrixResponseError("`room_id` not in response.")
 
+    fill_member_event_callback: Optional[Callable[[RoomID, UserID, MemberStateEventContent],
+                                                  Awaitable[Optional[MemberStateEventContent]]]]
+
+    async def fill_member_event(self, room_id: RoomID, user_id: UserID,
+                                content: MemberStateEventContent
+                                ) -> Optional[MemberStateEventContent]:
+        """
+        Fill a membership event content that is going to be sent in :meth:`send_member_event`.
+
+        This is used to set default fields like the displayname and avatar, which are usually set
+        by the server in the sugar membership endpoints like /join and /invite, but are not set
+        automatically when sending member events manually.
+
+        This default implementation only calls :prop:`fill_member_event_callback`.
+
+        Args:
+            room_id: The room where the member event is going to be sent.
+            user_id: The user whose membership is changing.
+            content: The new member event content.
+
+        Returns:
+            The filled member event content.
+        """
+        if self.fill_member_event_callback is not None:
+            return await self.fill_member_event_callback(room_id, user_id, content)
+        return None
+
     async def send_member_event(self, room_id: RoomID, user_id: UserID, membership: Membership,
                                 extra_content: Optional[Dict[str, Any]] = None) -> EventID:
+        """
+        Send a membership event manually.
+
+        Args:
+            room_id: The room to send the event to.
+            user_id: The user whose membership to change.
+            membership: The membership status.
+            extra_content: Additional content to put in the member event.
+
+        Returns:
+            The event ID of the new member event.
+        """
         content = MemberStateEventContent(membership=membership)
         for key, value in extra_content.items():
             content[key] = value
+        content = await self.fill_member_event(room_id, user_id, content) or content
         return await self.send_state_event(room_id, EventType.ROOM_MEMBER,
                                            content=content, state_key=user_id)
 
