@@ -1,13 +1,13 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Union, Dict, Optional, Any, List
+from typing import Union, Dict, Optional, Any, Tuple
 
 from .abstract import StateStore
-from mautrix.types import (Member, MemberStateEventContent, Membership, PowerLevelStateEventContent,
-                           RoomEncryptionStateEventContent, RoomID, UserID)
+from mautrix.types import (Member, MemberStateEventContent, Membership, RoomID, UserID,
+                           PowerLevelStateEventContent, RoomEncryptionStateEventContent)
 
 
 class MemoryStateStore(StateStore):
@@ -71,23 +71,32 @@ class MemoryStateStore(StateStore):
         try:
             room_members[user_id].membership = membership
         except (KeyError, TypeError):
-            room_id[user_id] = Member(membership=membership)
+            room_members[user_id] = Member(membership=membership)
 
-    async def get_members(self, room_id: RoomID) -> Optional[List[UserID]]:
+    async def get_member_profiles(
+        self, room_id: RoomID,
+        memberships: Tuple[Membership, ...] = (Membership.JOIN, Membership.INVITE),
+    ) -> Optional[Dict[UserID, Member]]:
         try:
-            return [user_id for user_id, member in self.members[room_id]
-                    if (member.membership == Membership.JOIN
-                        or member.membership == Membership.INVITE)]
+            return {user_id: member for user_id, member in self.members[room_id].items()
+                    if member.membership in memberships}
         except KeyError:
             return None
 
     async def set_members(self, room_id: RoomID,
-                          members: Dict[UserID, Union[Member, MemberStateEventContent]]) -> None:
+                          members: Dict[UserID, Union[Member, MemberStateEventContent]],
+                          only_membership: Optional[Membership] = None) -> None:
+        old_members = {}
+        if only_membership is not None:
+            old_members = {user_id: member for user_id, member
+                           in self.members.get(room_id, {}).items()
+                           if member.membership != only_membership}
         self.members[room_id] = {user_id: (member if isinstance(member, Member)
                                            else Member(membership=member.membership,
                                                        avatar_url=member.avatar_url,
                                                        displayname=member.displayname))
                                  for user_id, member in members.items()}
+        self.members[room_id].update(old_members)
         self.full_member_list[room_id] = True
 
     async def has_full_member_list(self, room_id: RoomID) -> bool:
