@@ -1,9 +1,9 @@
-# Copyright (c) 2020 Tulir Asokan
+# Copyright (c) 2021 Tulir Asokan
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional, Iterable, Dict
+from typing import Optional, Iterable, Dict, Tuple
 
 from sqlalchemy import Column, Text, Enum
 
@@ -31,17 +31,18 @@ class UserProfile(Base):
         return cls._select_one_or_none((cls.c.room_id == room_id) & (cls.c.user_id == user_id))
 
     @classmethod
-    def all_in_room(cls, room_id: RoomID, prefix: str = None, suffix: str = None, bot: str = None
-                    ) -> Iterable['UserProfile']:
-        clause = (((cls.c.membership == Membership.JOIN)
-                   | (cls.c.membership == Membership.INVITE))
-                  & (cls.c.room_id == room_id))
+    def all_in_room(cls, room_id: RoomID, memberships: Tuple[Membership, ...], prefix: str = None,
+                    suffix: str = None, bot: str = None) -> Iterable['UserProfile']:
+        clause = cls.c.membership == memberships[0]
+        for membership in memberships[1:]:
+            clause |= cls.c.membership == membership
+        clause &= cls.c.room_id == room_id
         if bot:
-            clause = clause & (cls.c.user_id != bot)
+            clause &= cls.c.user_id != bot
         if prefix:
-            clause = clause & ~cls.c.user_id.startswith(prefix, autoescape=True)
+            clause &= ~cls.c.user_id.startswith(prefix, autoescape=True)
         if suffix:
-            clause = clause & ~cls.c.user_id.startswith(suffix, autoescape=True)
+            clause &= ~cls.c.user_id.startswith(suffix, autoescape=True)
         return cls._select_all(clause)
 
     @classmethod
@@ -56,9 +57,12 @@ class UserProfile(Base):
             conn.execute(cls.t.delete().where(cls.c.room_id == room_id))
 
     @classmethod
-    def bulk_replace(cls, room_id: RoomID, members: Dict[UserID, Member]) -> None:
+    def bulk_replace(cls, room_id: RoomID, members: Dict[UserID, Member],
+                     only_membership: Optional[Membership] = None) -> None:
         with cls.db.begin() as conn:
             delete_condition = cls.c.room_id == room_id
+            if only_membership is not None:
+                delete_condition &= cls.c.membership == only_membership
             cls.db.execute(cls.t.delete().where(delete_condition))
             conn.execute(cls.t.insert(),
                          [dict(room_id=room_id, user_id=user_id, membership=member.membership,
