@@ -160,8 +160,12 @@ class Syncer(ABC):
         """
         if not isinstance(event_type, (EventType, InternalEventType)):
             raise ValueError("Invalid event type")
-        handler_list = (self.global_event_handlers if event_type == EventType.ALL
-                        else self.event_handlers[event_type])
+        try:
+            handler_list = (self.global_event_handlers if event_type == EventType.ALL
+                            else self.event_handlers[event_type])
+        except KeyError:
+            # No handlers for this event type registered
+            return
 
         # FIXME this is a bit hacky
         with suppress(ValueError):
@@ -279,7 +283,6 @@ class Syncer(ABC):
             # These aren't required by the spec, so make sure they're set
             raw_invite.setdefault("event_id", None)
             raw_invite.setdefault("origin_server_ts", int(time() * 1000))
-            raw_invite.setdefault("unsigned", {})
 
             invite = StateEvent.deserialize(raw_invite)
             invite.unsigned.invite_room_state = [StrippedStateEvent.deserialize(raw_event)
