@@ -1,6 +1,6 @@
 from .account import OlmAccount
 from .sessions import Session, InboundGroupSession, OutboundGroupSession
 from .types import TrustState, DeviceIdentity, DecryptedOlmEvent
-from .store import CryptoStore, StateStore, PgCryptoStore, PickleCryptoStore, MemoryCryptoStore
+from .store import CryptoStore, StateStore, PgCryptoStore, MemoryCryptoStore, PgCryptoStateStore
 from .machine import OlmMachine
 from .key_share import RejectKeyShare
