@@ -1,7 +1,6 @@
 from .abstract import CryptoStore, StateStore
 from .memory import MemoryCryptoStore
-from .pickle import PickleCryptoStore
 try:
-    from .asyncpg import PgCryptoStore
+    from .asyncpg import PgCryptoStore, PgCryptoStateStore
 except ImportError:
-    PgCryptoStore = None
+    PgCryptoStore = PgCryptoStateStore = None
