@@ -1,7 +1,7 @@
 from .primitive import (UserID, EventID, RoomID, RoomAlias, FilterID, ContentURI, SyncToken,
                         DeviceID, SessionID, SigningKey, IdentityKey, JSON)
 from .filter import Filter, EventFilter, RoomFilter, StateFilter, RoomEventFilter
-from .event import (EventType, GenericEvent,
+from .event import (EventType, GenericEvent, BaseRoomEvent, BaseEvent, BaseUnsigned,
 
                     RedactionEvent, RedactionEventContent,
                     ReactionEventContent, ReactionEvent,
@@ -46,7 +46,7 @@ from .users import User, Member, UserSearchResults
 from .auth import (LoginType, UserIdentifierType, MatrixUserIdentifier, ThirdPartyIdentifier,
                    PhoneIdentifier, UserIdentifier, LoginResponse, DiscoveryInformation,
                    DiscoveryServer, DiscoveryIntegrations, DiscoveryIntegrationServer,
-                   LoginFlow, LoginFlowList)
+                   LoginFlow, LoginFlowList, WhoamiResponse)
 from .crypto import UnsignedDeviceInfo, DeviceKeys, ClaimKeysResponse, QueryKeysResponse
 from .media import MediaRepoConfig, MXOpenGraph, OpenGraphVideo, OpenGraphImage, OpenGraphAudio
 from .util import (Obj, Lst, SerializerError, Serializable, SerializableEnum, SerializableAttrs,
