@@ -10,7 +10,7 @@ import re
 from attr import dataclass
 import attr
 
-from ..util import ExtensibleEnum, SerializableAttrs, Serializable, Obj, deserializer
+from ..util import ExtensibleEnum, SerializableAttrs, Serializable, Obj, deserializer, field
 from ..primitive import JSON, ContentURI, EventID
 from .base import BaseRoomEvent, BaseUnsigned
 
@@ -342,9 +342,10 @@ class TextMessageEventContent(BaseMessageEventContent,
         return None
 
     def trim_reply_fallback(self) -> None:
-        if self.get_reply_to():
+        if self.get_reply_to() and not getattr(self, "__reply_fallback_trimmed", False):
             self._trim_reply_fallback_text()
             self._trim_reply_fallback_html()
+            setattr(self, "__reply_fallback_trimmed", True)
 
     def _trim_reply_fallback_text(self) -> None:
         if not self.body.startswith("> ") or "\n" not in self.body:
@@ -393,7 +394,7 @@ media_reply_fallback_body_map = {
 class MessageEvent(BaseRoomEvent, SerializableAttrs):
     """An m.room.message event"""
     content: MessageEventContent
-    unsigned: Optional[MessageUnsigned] = None
+    unsigned: Optional[MessageUnsigned] = field(factory=lambda: MessageUnsigned())
 
     @staticmethod
     @deserializer(MessageEventContent)
