@@ -11,7 +11,7 @@ from ..util import SerializableEnum, ExtensibleEnum, Serializable
 
 
 class RoomType(ExtensibleEnum):
-    SPACE = "org.matrix.msc1772.space"
+    SPACE = "m.space"
 
 
 class EventType(Serializable):
@@ -122,8 +122,8 @@ _standard_types = {
         "m.room.tombstone": "ROOM_TOMBSTONE",
         "m.room.encryption": "ROOM_ENCRYPTION",
 
-        "org.matrix.msc1772.space.child": "SPACE_CHILD",
-        "org.matrix.msc1772.space.parent": "SPACE_PARENT",
+        "m.space.child": "SPACE_CHILD",
+        "m.space.parent": "SPACE_PARENT",
     },
     EventType.Class.MESSAGE: {
         "m.room.redaction": "ROOM_REDACTION",
@@ -131,6 +131,14 @@ _standard_types = {
         "m.room.encrypted": "ROOM_ENCRYPTED",
         "m.sticker": "STICKER",
         "m.reaction": "REACTION",
+
+        "m.call.invite": "CALL_INVITE",
+        "m.call.candidates": "CALL_CANDIDATES",
+        "m.call.select_answer": "CALL_SELECT_ANSWER",
+        "m.call.answer": "CALL_ANSWER",
+        "m.call.hangup": "CALL_HANGUP",
+        "m.call.reject": "CALL_REJECT",
+        "m.call.negotiate": "CALL_NEGOTIATE",
     },
     EventType.Class.EPHEMERAL: {
         "m.receipt": "RECEIPT",
